--[[ LibTransition-1.0
Revision: $Rev: 8 $
Author(s): Humbedooh
Description: PowerPoint goodness for your addon.
Dependencies: LibStub
License: All Rights Reserved.
]]

local tinsert,tremove,pow,sqrt,atan2,abs,floor,min,max,deg = table.insert, table.remove,math.pow,math.sqrt,math.atan2,math.abs,math.floor,math.min,math.max,math.deg;
local gravity, altitude, elasticity, jiffy = 9.81, 20, 9.81/20, 0.04;
local LITRANSITION = "LibTransition-1.0"
local LITRANSITION_MINOR = tonumber(("$Rev: 8 $"):match("(%d+)")) or 5000;
assert(LibStub, LITRANSITION .. " requires LibStub.")
local LibTransition = LibStub:NewLibrary(LITRANSITION, LITRANSITION_MINOR)
if not LibTransition then return end
LibTransition.methods = {
	FADEIN = {'duration'},
	FADEOUT = {'duration'}
};
LibTransition.tQueue = {};

function LibTransition:GetFrame(frame)
	if ( type(frame) == "string" ) then frame = _G[frame]; end
	assert(type(frame) == "table" and frame.Show, LITRANSITION .. ": Could not find frame object");
	return frame;
end

function LibTransition.runTimer(me, elapsed)
	local now = GetTime();
	for frame, queue in pairs(LibTransition.tQueue) do
		if ( #queue == 0 ) then
			LibTransition.tQueue[frame] = nil;
		else
			local when = queue[1].executeTime or (now + 1);
			while (when <= now ) do
				local x,y,ref,alpha,width,height = unpack((tremove(queue,1)).args);
				if x and y and ref then frame:SetPoint(ref, UIParent, "BOTTOMLEFT", x, y) end
				if alpha then
					if alpha == 0 then frame:Hide() else frame:Show() frame:SetAlpha(alpha) end
				end
				if width and height then frame:SetWidth(width) frame:SetHeight(height) end
				if ( #queue == 0 ) then break end
				when = queue[1].executeTime or (now + 1);
			end
		end
	end
end

function LibTransition:Queue(frame, method, ...)
	self:Compile(frame, method:upper(), ...);
end

function LibTransition:Compile(frame, method, ...)
	local now = GetTime();
	if not LibTransition.tQueue[frame] then LibTransition.tQueue[frame] = {};
	else now = LibTransition.tQueue[frame][#LibTransition.tQueue[frame]].executeTime + jiffy; end
	if ( method == "DROP" ) then
		local bounce = select(1, ...);
		local distance = altitude * (frame:GetBottom() / 768);
		local left = frame:GetLeft();
		local speed = 1;
		local duration = 0;
		local xduration = 0;
		while ( true ) do
			duration = duration + jiffy;
			xduration = xduration + jiffy;
			speed = speed + (jiffy*gravity);
			distance = distance - (speed * jiffy);
			if ( distance <= 0 ) then
				distance = 0;
			end
			tinsert(LibTransition.tQueue[frame], {args={left, (distance/altitude)*768, "BOTTOMLEFT"}, executeTime=now+xduration});
			if ( #(LibTransition.tQueue[frame]) == 300 ) then print("too much"); break; end
			if distance == 0 then
				if ( bounce ) then
					speed = speed * -elasticity;
					duration = 0;
					if abs(speed) <= 1 then break end
				end
			end
		end
	elseif ( method == "FADEIN" ) then
		local duration = abs(tonumber(select(1,...)) or 1);
		local elapsed = 0;
		while duration > elapsed do
			tinsert(LibTransition.tQueue[frame], {args={nil,nil,nil,elapsed/duration}, executeTime=now+elapsed});
			elapsed = elapsed + jiffy
		end
	elseif ( method == "FADEOUT" ) then
		local duration = abs(tonumber(select(1,...)) or 1);
		local elapsed = 0;
		while duration > elapsed do
			tinsert(LibTransition.tQueue[frame], {args={nil,nil,nil,1 - (elapsed/duration)}, executeTime=now+elapsed});
			elapsed = elapsed + jiffy
		end
		tinsert(LibTransition.tQueue[frame], {args={nil,nil,nil,0}, executeTime=now+elapsed});
	elseif ( method == "WAIT" ) then
		local duration = abs(tonumber(select(1,...)) or 1);
		tinsert(LibTransition.tQueue[frame], {args={}, executeTime=now+duration});
	end

end

function LibTransition:Run(frame)
end

function LibTransition:Attach(frame)
	frame = self:GetFrame(frame);
	frame._LibTransition = self;
	for name, func in pairs(self) do
		local xfunc = name:match("Shorthand_(.*)");
		if xfunc then frame[xfunc] = func; end
	end
end

function LibTransition:Shorthand_FadeIn(...) self._LibTransition:Queue(self, "FadeIn", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_FadeOut(...) self._LibTransition:Queue(self, "FadeOut", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_Cut(...) self._LibTransition:Queue(self, "Cut", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_WipeDown(...) self._LibTransition:Queue(self, "WipeDown", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_WipeUp(...) self._LibTransition:Queue(self, "WipeUp", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_BoxIn(...) self._LibTransition:Queue(self, "BoxIn", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_BoxOut(...) self._LibTransition:Queue(self, "BoxOut", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_Materialize(...) self._LibTransition:Queue(self, "Materialize", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_Dissolve(...) self._LibTransition:Queue(self, "Dissolve", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_Squeeze(...) self._LibTransition:Queue(self, "Squeeze", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_Stretch(...) self._LibTransition:Queue(self, "Stretch", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_PushIn(...) self._LibTransition:Queue(self, "PushIn", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_PushOut(...) self._LibTransition:Queue(self, "PushOut", ...) self._LibTransition:Run(self) end
function LibTransition:Shorthand_Cover(...) self._LibTransition:Queue(self, "Cover", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_Random(...) self._LibTransition:Queue(self, "Random", ...) self._LibTransition:Run(self) end
function LibTransition:Shorthand_Fly(...) self._LibTransition:Queue(self, "Fly", ...) self._LibTransition:Run(self)end
function LibTransition:Shorthand_Drop(...) self._LibTransition:Queue(self, "Drop", ...) self._LibTransition:Run(self) end
function LibTransition:Shorthand_Bungee(...) self._LibTransition:Queue(self, "Bungee", ...) self._LibTransition:Run(self) end
function LibTransition:Shorthand_Wait(...) self._LibTransition:Queue(self, "Wait", ...) self._LibTransition:Run(self) end

function libt_test()
	local frame = CreateFrame("Frame", UIParent);
	frame:SetPoint("CENTER",0,300);
	frame:SetWidth(100);
	frame:SetHeight(100);
	local tex = frame:CreateTexture("ARTWORK");
	tex:SetAllPoints(frame);
	tex:SetTexture(1,0,1,0.75);
	LibTransition:Attach(frame);
	frame:FadeIn(1);
	frame:Wait(3);
	frame:Drop(true);
	frame:FadeOut(1);
end

LibTransition.timer = CreateFrame("Frame", UIParent);
LibTransition.timer:SetPoint("BOTTOMRIGHT", 0, 0);
LibTransition.timer:SetWidth(1);
LibTransition.timer:SetHeight(1);
LibTransition.timer:SetScript("OnUpdate", LibTransition.runTimer);
